@@ -181,6 +181,7 @@ The *features* that CLM adds itself are:
 			      #+clisp " -fPIC"
 			      #+(or netbsd x86-64) " -fPIC"
 			      #+(and sgi acl-50) " -n32 -w"
+			      #+(and excl linux) " -fPIC"        ; 23-Mar-2018
 			      #+(and excl freebsd) " -fPIC -DPIC"
 			      ))
 
@@ -676,3 +677,4 @@ The *features* that CLM adds itself are:
   
 ;;; see README.clm for current status
 
+(clm-initialize-links)
