@@ -524,7 +524,7 @@ static void display_sound_file_entry(FILE *fp, const char *name, sound_file *sf)
 	      if (i > 1) fprintf(fp, ", ");
 	      fprintf(fp, " %.3f at %.3f ",
 		      sf->maxamps[i],
-		      (sf->srate > 0) ? (float)((double)(sf->maxtimes[i]) / (double)(sf->srate)) : (float)(sf->maxtimes[i]));
+		      (sf->srate > 0) ? ((double)(sf->maxtimes[i]) / (double)(sf->srate)) : (double)(sf->maxtimes[i]));
 	    }
 	}
     }
@@ -821,7 +821,7 @@ float mus_sound_duration(const char *arg)
   if (sf) 
     {
       if ((sf->chans > 0) && (sf->srate > 0))
-	val = (float)((double)(sf->samples) / ((float)(sf->chans) * (float)(sf->srate)));
+	val = ((double)(sf->samples) / ((double)(sf->chans) * (double)(sf->srate)));
       else val = 0.0;
     }
   return(val);
@@ -934,7 +934,7 @@ int mus_sound_mark_info(const char *arg, int **mark_ids, int **mark_positions)
 char *mus_sound_comment(const char *name)
 {
   char *sc = NULL;
-  sound_file *sf = NULL;
+  sound_file *sf;
   sf = get_sf(name); 
   if (sf)
     {
@@ -1007,7 +1007,7 @@ int mus_sound_open_input(const char *arg)
     mus_error(MUS_CANT_OPEN_FILE, "mus_sound_open_input: can't open %s: %s", arg, STRERROR(errno));
   else
     {
-      sound_file *sf = NULL;
+      sound_file *sf;
       mus_sound_initialize();
       sf = get_sf(arg);
       if (sf) 
@@ -1210,7 +1210,7 @@ bool mus_sound_maxamp_exists(const char *ifile)
 mus_long_t mus_sound_maxamps(const char *ifile, int chans, mus_float_t *vals, mus_long_t *times)
 {
   mus_long_t framples;
-  unsigned int ichans, chn;
+  uint32_t ichans, chn;
   sound_file *sf; 
     
   sf = get_sf(ifile); 
@@ -1221,7 +1221,7 @@ mus_long_t mus_sound_maxamps(const char *ifile, int chans, mus_float_t *vals, mu
     {
       if (chans > sf->maxamps_size) 
 	ichans = sf->maxamps_size; 
-      else ichans = (unsigned int)chans;
+      else ichans = (uint32_t)chans;
       for (chn = 0; chn < ichans; chn++)
 	{
 	  times[chn] = sf->maxtimes[chn];
@@ -1240,7 +1240,7 @@ mus_long_t mus_sound_maxamps(const char *ifile, int chans, mus_float_t *vals, mu
 
     ifd = mus_sound_open_input(ifile);
     if (ifd == MUS_ERROR) return(MUS_ERROR);
-    ichans = (unsigned int)mus_sound_chans(ifile);
+    ichans = (uint32_t)mus_sound_chans(ifile);
     framples = mus_sound_framples(ifile);
     if ((framples == 0) || (ichans > MUS_MAX_CHANS))
       {
@@ -1439,7 +1439,7 @@ const char *mus_array_to_file_with_error(const char *filename, mus_float_t *ddat
 {
   /* put ddata into a sound file, taking byte order into account */
   /* assume ddata is interleaved already if more than one channel */
-  int fd, err = MUS_NO_ERROR;
+  int fd, err;
   mus_long_t oloc;
   mus_sound_forget(filename);
 
@@ -1497,7 +1497,7 @@ int mus_sound_initialize(void)
   static bool sndlib_initialized = false;
   if (!sndlib_initialized)
     {
-      int err = MUS_NO_ERROR;
+      int err;
       sndlib_initialized = true;
       mus_error_handler = default_mus_error;
       err = mus_header_initialize();
