@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; Data object ref/set stuff.
 
@@ -35,7 +35,6 @@
   (:info name dx-p offset lowtag)
   (:ignore name dx-p))
 
-#!+compare-and-swap-vops
 (define-vop (compare-and-swap-slot)
   (:args (object :scs (descriptor-reg))
          (old :scs (descriptor-reg any-reg))
@@ -59,7 +58,6 @@
 
 ;;;; Symbol hacking VOPs:
 
-#!+compare-and-swap-vops
 (define-vop (%compare-and-swap-symbol-value)
   (:translate %compare-and-swap-symbol-value)
   (:args (symbol :scs (descriptor-reg))
@@ -71,7 +69,7 @@
   (:vop-var vop)
   (:generator 15
     (inst sync)
-    #!+sb-thread
+    #+sb-thread
     (assemble ()
       (loadw temp symbol symbol-tls-index-slot other-pointer-lowtag)
       ;; Thread-local area, no synchronization needed.
@@ -127,7 +125,7 @@
   (:policy :fast)
   (:translate sym-global-val))
 
-#!+sb-thread
+#+sb-thread
 (progn
   (define-vop (set)
     (:args (symbol :scs (descriptor-reg))
@@ -180,7 +178,7 @@
       DONE)))
 
 ;;; On unithreaded builds these are just copies of the global versions.
-#!-sb-thread
+#-sb-thread
 (progn
   (define-vop (symbol-value symbol-global-value)
     (:translate symeval))
@@ -197,7 +195,7 @@
   (:policy :fast-safe)
   (:temporary (:scs (descriptor-reg)) value))
 
-#!+sb-thread
+#+sb-thread
 (define-vop (boundp boundp-frob)
   (:translate boundp)
   (:generator 9
@@ -210,7 +208,7 @@
     (inst cmpwi value unbound-marker-widetag)
     (inst b? (if not-p :eq :ne) target)))
 
-#!-sb-thread
+#-sb-thread
 (define-vop (boundp boundp-frob)
   (:translate boundp)
   (:generator 9
@@ -266,9 +264,9 @@
       (inst cmpwi type simple-fun-widetag)
       ;;(inst mr lip function)
       (inst addi lip function
-            (- (ash simple-fun-code-offset word-shift) fun-pointer-lowtag))
+            (- (ash simple-fun-insts-offset word-shift) fun-pointer-lowtag))
       (inst beq normal-fn)
-      (inst lr lip (make-fixup 'closure-tramp :assembly-routine))
+      (load-asm-rtn-addr lip 'closure-tramp)
       (emit-label normal-fn)
       (storew lip fdefn fdefn-raw-addr-slot other-pointer-lowtag)
       (storew function fdefn fdefn-fun-slot other-pointer-lowtag)
@@ -282,7 +280,7 @@
   (:results (result :scs (descriptor-reg)))
   (:generator 38
     (storew null-tn fdefn fdefn-fun-slot other-pointer-lowtag)
-    (inst lr temp (make-fixup 'undefined-tramp :assembly-routine))
+    (load-asm-rtn-addr temp 'undefined-tramp)
     (storew temp fdefn fdefn-raw-addr-slot other-pointer-lowtag)
     (move result fdefn)))
 
@@ -294,7 +292,7 @@
 ;;; the symbol on the binding stack and stuff the new value into the
 ;;; symbol.
 ;;; See the "Chapter 9: Specials" of the SBCL Internals Manual.
-#!+sb-thread
+#+sb-thread
 (define-vop (dynbind)
   (:args (val :scs (any-reg descriptor-reg))
          (symbol :scs (descriptor-reg)))
@@ -351,7 +349,7 @@
      (storew tls-index bsp-tn (- binding-symbol-slot binding-size))
      (inst stwx val thread-base-tn tls-index)))
 
-#!-sb-thread
+#-sb-thread
 (define-vop (dynbind)
   (:args (val :scs (any-reg descriptor-reg))
          (symbol :scs (descriptor-reg)))
@@ -363,7 +361,7 @@
     (storew symbol bsp-tn (- binding-symbol-slot binding-size))
     (storew val symbol symbol-value-slot other-pointer-lowtag)))
 
-#!+sb-thread
+#+sb-thread
 (define-vop (unbind)
   (:temporary (:scs (descriptor-reg)) tls-index value)
   (:generator 0
@@ -374,7 +372,7 @@
     (storew zero-tn bsp-tn (- binding-value-slot binding-size))
     (inst subi bsp-tn bsp-tn (* binding-size n-word-bytes))))
 
-#!-sb-thread
+#-sb-thread
 (define-vop (unbind)
   (:temporary (:scs (descriptor-reg)) symbol value)
   (:generator 0
@@ -403,9 +401,9 @@
       (inst cmpwi symbol 0)
       (inst beq skip)
       (loadw value bsp-tn (- binding-value-slot binding-size))
-      #!+sb-thread
+      #+sb-thread
       (inst stwx value thread-base-tn symbol)
-      #!-sb-thread
+      #-sb-thread
       (storew value symbol symbol-value-slot other-pointer-lowtag)
       (storew zero-tn bsp-tn (- binding-symbol-slot binding-size))
 
@@ -433,11 +431,19 @@
   (:variant funcallable-instance-info-offset fun-pointer-lowtag)
   (:translate %set-funcallable-instance-info))
 
-(define-vop (closure-ref slot-ref)
-  (:variant closure-info-offset fun-pointer-lowtag))
+(define-vop (closure-ref)
+  (:args (object :scs (descriptor-reg)))
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:info offset)
+  (:generator 4
+    (loadw value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
-(define-vop (closure-init slot-set)
-  (:variant closure-info-offset fun-pointer-lowtag))
+(define-vop (closure-init)
+  (:args (object :scs (descriptor-reg))
+         (value :scs (descriptor-reg any-reg)))
+  (:info offset)
+  (:generator 4
+    (storew value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
 (define-vop (closure-init-from-fp)
   (:args (object :scs (descriptor-reg)))
@@ -466,7 +472,9 @@
   (:result-types positive-fixnum)
   (:generator 4
     (loadw temp struct 0 instance-pointer-lowtag)
-    (inst srwi res temp n-widetag-bits)))
+    ;; shift right 8 and mask 15 low bits =
+    ;; rotate left 24, take bit indices 17 through 31.
+    (inst rlwinm res temp (- 32 n-widetag-bits) 17 31)))
 
 (define-vop (instance-index-ref word-index-ref)
   (:policy :fast-safe)
@@ -480,7 +488,6 @@
   (:variant instance-slots-offset instance-pointer-lowtag)
   (:arg-types instance positive-fixnum *))
 
-#!+compare-and-swap-vops
 (define-vop (%instance-cas word-index-cas)
   (:policy :fast-safe)
   (:translate %instance-cas)
@@ -508,13 +515,31 @@
   (- (ash (+ index displacement instance-slots-offset) word-shift)
      instance-pointer-lowtag))
 
-(define-vop (raw-instance-init/word)
-  (:args (object :scs (descriptor-reg))
-         (value :scs (unsigned-reg)))
-  (:arg-types * unsigned-num)
-  (:info index)
-  (:generator 4
-    (inst stw value object (offset-for-raw-slot index))))
+(macrolet ((def (suffix sc primtype)
+             `(progn
+                (define-vop (,(symbolicate "RAW-INSTANCE-INIT/" suffix))
+                  (:args (object :scs (descriptor-reg))
+                         (value :scs (,sc)))
+                  (:arg-types * ,primtype)
+                  (:info index)
+                  (:generator 4 (inst stw value object (offset-for-raw-slot index))))
+                (define-vop (,(symbolicate "RAW-INSTANCE-REF/" suffix) word-index-ref)
+                  (:policy :fast-safe)
+                  (:translate ,(symbolicate "%RAW-INSTANCE-REF/" suffix))
+                  (:variant instance-slots-offset instance-pointer-lowtag)
+                  (:arg-types instance positive-fixnum)
+                  (:results (value :scs (,sc)))
+                  (:result-types ,primtype))
+                (define-vop (,(symbolicate "RAW-INSTANCE-SET/" suffix) word-index-set)
+                  (:policy :fast-safe)
+                  (:translate ,(symbolicate "%RAW-INSTANCE-SET/" suffix))
+                  (:variant instance-slots-offset instance-pointer-lowtag)
+                  (:arg-types instance positive-fixnum ,primtype)
+                  (:args (object) (index) (value :scs (,sc)))
+                  (:results (result :scs (,sc)))
+                  (:result-types ,primtype)))))
+  (def word unsigned-reg unsigned-num)
+  (def signed-word signed-reg signed-num))
 
 (define-vop (raw-instance-atomic-incf/word)
   (:translate %raw-instance-atomic-incf/word)
@@ -541,23 +566,6 @@
     (inst bne LOOP)
     (inst isync)))
 
-(define-vop (raw-instance-ref/word word-index-ref)
-  (:policy :fast-safe)
-  (:translate %raw-instance-ref/word)
-  (:variant instance-slots-offset instance-pointer-lowtag)
-  (:arg-types instance positive-fixnum)
-  (:results (value :scs (unsigned-reg)))
-  (:result-types unsigned-num))
-
-(define-vop (raw-instance-set/word word-index-set)
-  (:policy :fast-safe)
-  (:translate %raw-instance-set/word)
-  (:variant instance-slots-offset instance-pointer-lowtag)
-  (:arg-types instance positive-fixnum unsigned-num)
-  (:args (object) (index) (value :scs (unsigned-reg)))
-  (:results (result :scs (unsigned-reg)))
-  (:result-types unsigned-num))
-
 (define-vop (raw-instance-init/single)
   (:args (object :scs (descriptor-reg))
          (value :scs (single-reg)))
